@@ -1,6 +1,6 @@
 ;;; wisi-process-parse.el --- interface to external parse program
 ;;
-;; Copyright (C) 2014, 2017 - 2019 Free Software Foundation, Inc.
+;; Copyright (C) 2014, 2017 - 2020 Free Software Foundation, Inc.
 ;;
 ;; Author: Stephen Leake <stephen_leake@member.fsf.org>
 ;;
@@ -31,10 +31,10 @@
   is no response from the parser after waiting this amount (in
   seconds)."
   :type 'float
-  :safe 'floatp)
+  :safe 'numberp)
 (make-variable-buffer-local 'wisi-process-time-out)
 
-(defconst wisi-process-parse-protocol-version "4"
+(defconst wisi-process-parse-protocol-version "5"
   "Defines data exchanged between this package and the background process.
 Must match emacs_wisi_common_parse.ads Protocol_Version.")
 
@@ -83,7 +83,8 @@ Otherwise add PARSER to ‘wisi-process--alist’, return it."
       (let ((exec-file (locate-file (wisi-process--parser-exec-file parser) exec-path '("" ".exe"))))
 
 	(unless exec-file
-	  (error "%s not found on `exec-path'" (wisi-process--parser-exec-file parser)))
+	  (error "%s not found on `exec-path'; run 'build.sh' in the ELPA package."
+		 (wisi-process--parser-exec-file parser)))
 
 	(push (cons (wisi-process--parser-label parser) parser) wisi-process--alist)
 
@@ -103,17 +104,20 @@ Otherwise add PARSER to ‘wisi-process--alist’, return it."
   ;; process buffer contains the process and language protocol versions.
   (with-current-buffer (wisi-process--parser-buffer parser)
     (goto-char (point-min))
-    (search-forward-regexp "protocol: process version \\([0-9]+\\) language version \\([0-9]+\\)")
-    (unless (and (match-string 1)
-		 (string-equal (match-string 1) wisi-process-parse-protocol-version)
-		 (match-string 2)
-		 (string-equal (match-string 2) (wisi-process--parser-language-protocol-version parser)))
-      (wisi-parse-kill parser)
-      (error "%s parser process protocol version mismatch: elisp %s %s, process %s %s"
-	     (wisi-process--parser-label parser)
-	     wisi-process-parse-protocol-version (wisi-process--parser-language-protocol-version parser)
-	     (match-string 1) (match-string 2)))
-    ))
+    (if (search-forward-regexp "protocol: process version \\([0-9]+\\) language version \\([0-9]+\\)" nil t)
+	(unless (and (match-string 1)
+		     (string-equal (match-string 1) wisi-process-parse-protocol-version)
+		     (match-string 2)
+		     (string-equal (match-string 2) (wisi-process--parser-language-protocol-version parser)))
+	  (wisi-parse-kill parser)
+	  (error "%s parser process protocol version mismatch: elisp %s %s, process %s %s"
+		 (wisi-process--parser-label parser)
+		 wisi-process-parse-protocol-version (wisi-process--parser-language-protocol-version parser)
+		 (match-string 1) (match-string 2)))
+      ;; Search failed
+      (error "%s parser process protocol version message not found"
+	     (wisi-process--parser-label parser))
+    )))
 
 (defun wisi-process-parse--require-process (parser)
   "Start the process for PARSER if not already started."
@@ -137,7 +141,6 @@ Otherwise add PARSER to ‘wisi-process--alist’, return it."
 			      (wisi-process--parser-exec-opts parser))))
 
       (set-process-query-on-exit-flag (wisi-process--parser-process parser) nil)
-      (setf (wisi-process--parser-busy parser) nil)
 
       (wisi-process-parse--wait parser)
       (wisi-process-parse--check-version parser)
@@ -194,7 +197,7 @@ parse region."
 		      (if (or (and (= begin (point-min)) (= parse-end (point-max)))
 			      (< (point-max) wisi-partial-parse-threshold))
 			  0 1) ;; partial parse active
-		      (if (> wisi-debug 0) 1 0) ;; debug-mode
+		      (if (> wisi-debug 0) 1 0) ;; debug_mode
 		      (1- wisi-debug) ;; trace_parse
 		      wisi-trace-mckenzie
 		      wisi-trace-action
@@ -225,7 +228,7 @@ PARSE-END, wait for command to complete. PARSER will respond with
 one or more Edit messages."
   ;; Must match "refactor" command arguments read by
   ;; emacs_wisi_common_parse.adb Get_Refactor_Params.
-  (let* ((cmd (format "refactor %d \"%s\" %d %d %d %d %d %d %d %d %d %d %d"
+  (let* ((cmd (format "refactor %d \"%s\" %d %d %d %d %d %d %d %d %d %d %d %d"
 		      refactor-action
 		      (if (buffer-file-name) (file-name-nondirectory (buffer-file-name)) "")
 		      (position-bytes parse-begin)
@@ -234,6 +237,7 @@ one or more Edit messages."
 		      parse-begin ;; char_pos
 		      (line-number-at-pos parse-begin)
 		      (line-number-at-pos parse-end)
+		      (save-excursion (goto-char parse-begin) (back-to-indentation) (current-column));; indent-begin
 		      (if (> wisi-debug 0) 1 0) ;; debug-mode
 		      (1- wisi-debug) ;; trace_parse
 		      wisi-trace-action
@@ -396,7 +400,7 @@ complete."
       (cl-do ((i 1 (1+ i))) ((= i (length sexp)))
 	(push
 	 (make-wisi--parse-error-repair
-	  :pos (aref (aref sexp i) 0)
+	  :pos (copy-marker (aref (aref sexp i) 0))
 	  :inserted (mapcar (lambda (id) (aref token-table id)) (aref (aref sexp i) 1))
 	  :deleted  (mapcar (lambda (id) (aref token-table id)) (aref (aref sexp i) 2))
 	  :deleted-region (aref (aref sexp i) 3))
@@ -406,7 +410,7 @@ complete."
 (defun wisi-process-parse--End (parser sexp)
   ;; sexp is [End pos]
   ;; see ‘wisi-process-parse--execute’
-  (setf (wisi-process--parser-end-pos parser) (aref sexp 1)))
+  (setf (wisi-process--parser-end-pos parser) (1+ (aref sexp 1))))
 
 (defun wisi-process-parse--Edit (parser sexp)
   ;; sexp is [Edit begin end text]
@@ -516,8 +520,6 @@ complete."
 (declare-function wisi-elisp-lexer-reset "wisi-elisp-lexer")
 
 (defun wisi-process-parse--prepare (parser)
-  (wisi-process-parse--require-process parser)
-
   ;; font-lock can trigger a face parse while navigate or indent parse
   ;; is active, due to ‘accept-process-output’ below. Signaling an
   ;; error tells font-lock to try again later.
@@ -539,6 +541,13 @@ complete."
     ;; accept-process-output below.
     (setf (wisi-process--parser-busy parser) t)
 
+    ;; If the parser process has not started yet,
+    ;; wisi-process-parse--require-process calls
+    ;; wisi-process-parse--wait, which can let font-lock invoke the
+    ;; parser again. Thus this call must be after we set
+    ;; wisi-process--parser-busy t
+    (wisi-process-parse--require-process parser)
+
     (setf (wisi-process--parser-total-wait-time parser) 0.0)
     (setf (wisi-parser-lexer-errors parser) nil)
     (setf (wisi-parser-parse-errors parser) nil)
@@ -596,7 +605,7 @@ complete."
 		(set-buffer source-buffer) ;; for put-text-property in actions
 		(cond
 		 ((listp response)
-		  ;; error of some sort
+		  ;; non-syntax error of some sort
 		  (cond
 		   ((equal '(parse_error) response)
 		    ;; Parser detected a syntax error, and recovery failed, so signal it.
@@ -652,7 +661,7 @@ complete."
 		     (push (make-wisi--parse-error :pos (point) :message (cadr err)) (wisi-parser-parse-errors parser))
 		     (signal (car err) (cdr err)))
 
-		    (error ;; ie from [C:\Windows\system32\KERNEL32.DLL], or bug in action code above.
+		    (error ;; ie from un-commented [C:\Windows\system32\KERNEL32.DLL], or bug in action code above.
 		     (set-buffer response-buffer)
 		     (let ((content (buffer-substring-no-properties (point-min) (point-max)))
 			   (buf-name (concat (buffer-name) "-save-error")))
@@ -703,7 +712,7 @@ complete."
 
 	      (when (and (= (point-max) need-more)
 			 (> (wisi-process--parser-total-wait-time parser) wisi-process-time-out))
-		(error "wisi-process-parse not getting more text (or bad syntax in process output)"))
+		(error "wisi-process-parse timing out; increase `wisi-process-time-out'? (or bad syntax in process output)"))
 
 	      (setq need-more nil))
 	    );; while not done
