@@ -7,7 +7,7 @@
 ;; Keywords: parser
 ;;  indentation
 ;;  navigation
-;; Version: 3.1.1
+;; Version: 3.1.2
 ;; package-requires: ((emacs "25.0") (seq "2.20"))
 ;; URL: http://stephe-leake.org/ada/wisitoken.html
 ;;
@@ -468,14 +468,16 @@ Used to ignore whitespace changes in before/after change hooks.")
     ;; don't have to do it again in wisi-after-change.
     (setq wisi--change-beg (min wisi--change-beg begin))
 
+    ;; `buffer-base-buffer' deals with edits in indirect buffers
+    ;; created by ediff-regions-*
+
     (cond
      ((null wisi--change-end)
-      (setq wisi--change-end (copy-marker end)))
+      (setq wisi--change-end (make-marker))
+      (set-marker wisi--change-end end (or (buffer-base-buffer) (current-buffer))))
 
      ((> end wisi--change-end)
-      ;; `buffer-base-buffer' deals with edits in indirect buffers
-      ;; created by ediff-regions-*
-      (set-marker wisi--change-end end (buffer-base-buffer)))
+      (set-marker wisi--change-end end (or (buffer-base-buffer) (current-buffer))))
      )
 
     (unless (= begin end)
@@ -1660,6 +1662,7 @@ where the car is a list (FILE LINE COL)."
 (defun wisi-show-containing-or-previous-cache ()
   (interactive)
   (let ((cache (wisi-get-cache (point))))
+    (push-mark)
     (if cache
 	(message "containing %s" (wisi-goto-containing cache t))
       (message "previous %s" (wisi-backward-cache)))
